'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var _asyncToGenerator = require('@babel/runtime/helpers/asyncToGenerator');
var _classCallCheck = require('@babel/runtime/helpers/classCallCheck');
var _createClass = require('@babel/runtime/helpers/createClass');
var _regeneratorRuntime = require('@babel/runtime/regenerator');
var ptokensHelpers = require('ptokens-helpers');
var jsonrpc = require('jsonrpc-lite');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var _asyncToGenerator__default = /*#__PURE__*/_interopDefaultLegacy(_asyncToGenerator);
var _classCallCheck__default = /*#__PURE__*/_interopDefaultLegacy(_classCallCheck);
var _createClass__default = /*#__PURE__*/_interopDefaultLegacy(_createClass);
var _regeneratorRuntime__default = /*#__PURE__*/_interopDefaultLegacy(_regeneratorRuntime);
var jsonrpc__default = /*#__PURE__*/_interopDefaultLegacy(jsonrpc);

exports.Status = void 0;

(function (Status) {
  Status[Status["ERROR"] = -1] = "ERROR";
  Status[Status["DETECTED"] = 0] = "DETECTED";
  Status[Status["BROADCASTED"] = 1] = "BROADCASTED";
})(exports.Status || (exports.Status = {}));

var pTokensNode = /*#__PURE__*/function () {
  /**
   * Create and initialize a pTokensNode object.
   * @param _provider - A pTokensNodeProvider that will be employed to send requests.
   */
  function pTokensNode(_provider) {
    _classCallCheck__default["default"](this, pTokensNode);

    this._provider = _provider;
  }
  /** Return the pTokensNodeProvider set when calling constructor. */


  _createClass__default["default"](pTokensNode, [{
    key: "provider",
    get: function get() {
      return this._provider;
    }
    /**
     * Get the transaction status related to a transaction.
     * @param _txHash - The hash of the transaction.
     * @param _originatingChainId - The chain ID where the transaction took place.
     * @returns A Promise that resolves with a TransactionStatus object related to the queried transaction.
     */

  }, {
    key: "getTransactionStatus",
    value: function () {
      var _getTransactionStatus = _asyncToGenerator__default["default"]( /*#__PURE__*/_regeneratorRuntime__default["default"].mark(function _callee(_txHash, _originatingChainId) {
        return _regeneratorRuntime__default["default"].wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _context.next = 2;
                return this._provider.sendRpcRequest(1, 'node_getTransactionStatus', [_txHash, _originatingChainId]);

              case 2:
                return _context.abrupt("return", _context.sent);

              case 3:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function getTransactionStatus(_x, _x2) {
        return _getTransactionStatus.apply(this, arguments);
      }

      return getTransactionStatus;
    }()
    /**
     * Get asset info related to those blockchains where pNetwork supports the input symbol.
     * @param _tokenSymbol - The token symbol.
     * @returns An array of AssetInfo objects for those chains where pNetwork supports the specified token.
     */

  }, {
    key: "getSupportedChainsByAsset",
    value: function () {
      var _getSupportedChainsByAsset = _asyncToGenerator__default["default"]( /*#__PURE__*/_regeneratorRuntime__default["default"].mark(function _callee2(_tokenSymbol) {
        return _regeneratorRuntime__default["default"].wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                _context2.next = 2;
                return this._provider.sendRpcRequest(1, 'node_getSupportedChainsByAsset', [_tokenSymbol]);

              case 2:
                return _context2.abrupt("return", _context2.sent);

              case 3:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function getSupportedChainsByAsset(_x3) {
        return _getSupportedChainsByAsset.apply(this, arguments);
      }

      return getSupportedChainsByAsset;
    }()
    /**
     * Get asset info related to a particular blockchain where pNetwork should support the input symbol.
     * @param _tokenSymbol - The token symbol.
     * @param _chainId - The chain ID of the blockchain.
     * @returns An AssetInfo object for the specified blockchain. Null if the specified blockchain does not support the token.
     */

  }, {
    key: "getAssetInfoByChainId",
    value: function () {
      var _getAssetInfoByChainId = _asyncToGenerator__default["default"]( /*#__PURE__*/_regeneratorRuntime__default["default"].mark(function _callee3(_tokenSymbol, _chainId) {
        var info;
        return _regeneratorRuntime__default["default"].wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                _context3.next = 2;
                return this.getSupportedChainsByAsset(_tokenSymbol);

              case 2:
                info = _context3.sent.filter(function (p) {
                  return p.chainId == _chainId;
                });
                return _context3.abrupt("return", info.length ? info.at(0) : null);

              case 4:
              case "end":
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      function getAssetInfoByChainId(_x4, _x5) {
        return _getAssetInfoByChainId.apply(this, arguments);
      }

      return getAssetInfoByChainId;
    }()
    /**
     * Get a native deposit address where to transfer a UTXO asset for initiating a peg-in.
     * @param _originatingChainId - The chain ID of the source asset.
     * @param _address - The destination asset in the destination chain ID.
     * @param _destinationChainId - The chain ID of the destination asset.
     * @returns A Promise that resolves with a NativeDepositAddress.
     */

  }, {
    key: "getNativeDepositAddress",
    value: function () {
      var _getNativeDepositAddress = _asyncToGenerator__default["default"]( /*#__PURE__*/_regeneratorRuntime__default["default"].mark(function _callee4(_originatingChainId, _address, _destinationChainId) {
        return _regeneratorRuntime__default["default"].wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                _context4.next = 2;
                return this._provider.sendRpcRequest(1, 'node_getNativeDepositAddress', [_originatingChainId, _address, _destinationChainId]);

              case 2:
                return _context4.abrupt("return", _context4.sent);

              case 3:
              case "end":
                return _context4.stop();
            }
          }
        }, _callee4, this);
      }));

      function getNativeDepositAddress(_x6, _x7, _x8) {
        return _getNativeDepositAddress.apply(this, arguments);
      }

      return getNativeDepositAddress;
    }()
  }]);

  return pTokensNode;
}();

var pTokensNodeProvider = /*#__PURE__*/function () {
  /**
   * Create and initialize a pTokensNodeProvider object.
   * @param _url - The URL of a pNetwork node running pNetwork webserver v3.
   */
  function pTokensNodeProvider(_url) {
    _classCallCheck__default["default"](this, pTokensNodeProvider);

    this._url = _url;
  }
  /**
   * The URL set when creating the object.
   */


  _createClass__default["default"](pTokensNodeProvider, [{
    key: "url",
    get: function get() {
      return this._url;
    }
    /**
     * Send a RPC request to the pNetwork webserver.
     * @param _reqId - The request ID.
     * @param _method - The RPC method to be called.
     * @param _params - Optional parameters that will be passed to the RPC method.
     * @returns A Promise that resolves with the result of the JSON response.
     */

  }, {
    key: "sendRpcRequest",
    value: function () {
      var _sendRpcRequest = _asyncToGenerator__default["default"]( /*#__PURE__*/_regeneratorRuntime__default["default"].mark(function _callee(_reqId, _method, _params) {
        var req, resp;
        return _regeneratorRuntime__default["default"].wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                req = jsonrpc__default["default"].request(_reqId, _method, _params);
                _context.next = 3;
                return ptokensHelpers.http.fetchJsonByPost(this.url, req);

              case 3:
                resp = _context.sent;

                if (!('error' in resp)) {
                  _context.next = 8;
                  break;
                }

                throw new Error("JSON RPC error ".concat(resp.error.message));

              case 8:
                if (!resp.result) {
                  _context.next = 12;
                  break;
                }

                return _context.abrupt("return", resp.result);

              case 12:
                throw new Error("Invalid JSON RPC response ".concat(JSON.stringify(resp)));

              case 13:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function sendRpcRequest(_x, _x2, _x3) {
        return _sendRpcRequest.apply(this, arguments);
      }

      return sendRpcRequest;
    }()
  }]);

  return pTokensNodeProvider;
}();

exports.pTokensNode = pTokensNode;
exports.pTokensNodeProvider = pTokensNodeProvider;
